@@ -10,10 +10,9 @@ from .core.django import setup_django
 def register(_test: bool = False):
     """Register an instance on the hub."""
     from ._check_setup import _check_instance_setup
-    from .core._hub_core import init_instance as init_instance_hub
-    from .core._hub_core import init_storage as init_storage_hub
+    from .core._hub_core import init_instance_hub, init_storage_hub
 
-    logger.warning("""lamin register will be removed soon""")
+    logger.warning("note that register() is only for testing purposes")
 
     isettings = settings.instance
     if not _check_instance_setup() and not _test:
@@ -22,8 +21,12 @@ def register(_test: bool = False):
     ssettings = settings.instance.storage
     if ssettings._uid is None and _test:
         # because django isn't up, we can't get it from the database
-        ssettings._uid = base62(8)
-    init_storage_hub(ssettings)
+        ssettings._uid = base62(12)
     init_instance_hub(isettings)
+    init_storage_hub(ssettings, is_default=True)
     isettings._is_on_hub = True
     isettings._persist()
+    if isettings.dialect != "sqlite" and not _test:
+        from ._schema_metadata import update_schema_in_hub
+
+        update_schema_in_hub()
