@@ -11,7 +11,8 @@ Settings:
    StorageSettings
 
 """
-from . import django, types, upath
+
+from . import django, upath
 from ._deprecated import deprecated
 from ._docs import doc_args
 from ._settings import SetupSettings
